@@ -1,3 +1,20 @@
+/*
+ * Copyright (C) The c-ares project
+ *
+ * Permission to use, copy, modify, and distribute this
+ * software and its documentation for any purpose and without
+ * fee is hereby granted, provided that the above copyright
+ * notice appear in all copies and that both that copyright
+ * notice and this permission notice appear in supporting
+ * documentation, and that the name of M.I.T. not be used in
+ * advertising or publicity pertaining to distribution of the
+ * software without specific, written prior permission.
+ * M.I.T. makes no representations about the suitability of
+ * this software for any purpose.  It is provided "as is"
+ * without express or implied warranty.
+ *
+ * SPDX-License-Identifier: MIT
+ */
 #include "ares-test.h"
 
 // library initialization is only needed for windows builds
@@ -88,6 +105,8 @@ TEST_F(LibraryTest, OptionsChannelInit) {
   optmask |= ARES_OPT_ROTATE;
   opts.resolvconf_path = strdup("/etc/resolv.conf");
   optmask |= ARES_OPT_RESOLVCONF;
+  opts.hosts_path = strdup("/etc/hosts");
+  optmask |= ARES_OPT_HOSTS_FILE;
 
   ares_channel channel = nullptr;
   EXPECT_EQ(ARES_SUCCESS, ares_init_options(&channel, &opts, optmask));
@@ -115,6 +134,7 @@ TEST_F(LibraryTest, OptionsChannelInit) {
   EXPECT_EQ(std::string(opts.domains[1]), std::string(opts2.domains[1]));
   EXPECT_EQ(std::string(opts.lookups), std::string(opts2.lookups));
   EXPECT_EQ(std::string(opts.resolvconf_path), std::string(opts2.resolvconf_path));
+  EXPECT_EQ(std::string(opts.hosts_path), std::string(opts2.hosts_path));
 
   ares_destroy_options(&opts);
   ares_destroy_options(&opts2);
@@ -174,6 +194,8 @@ TEST_F(LibraryTest, OptionsChannelAllocFail) {
   optmask |= ARES_OPT_ROTATE;
   opts.resolvconf_path = strdup("/etc/resolv.conf");
   optmask |= ARES_OPT_RESOLVCONF;
+  opts.hosts_path = strdup("/etc/hosts");
+  optmask |= ARES_OPT_HOSTS_FILE;
 
   ares_channel channel = nullptr;
   for (int ii = 1; ii <= 8; ii++) {
@@ -270,6 +292,8 @@ TEST_F(DefaultChannelTest, SetAddresses) {
 
 TEST_F(DefaultChannelTest, SetSortlistFailures) {
   EXPECT_EQ(ARES_ENODATA, ares_set_sortlist(nullptr, "1.2.3.4"));
+  EXPECT_EQ(ARES_EBADSTR, ares_set_sortlist(channel_, "111.111.111.111*/16"));
+  EXPECT_EQ(ARES_EBADSTR, ares_set_sortlist(channel_, "111.111.111.111/255.255.255.240*"));
   EXPECT_EQ(ARES_SUCCESS, ares_set_sortlist(channel_, "xyzzy ; lwk"));
   EXPECT_EQ(ARES_SUCCESS, ares_set_sortlist(channel_, "xyzzy ; 0x123"));
 }
@@ -430,6 +454,32 @@ CONTAINED_TEST_F(LibraryTest, ContainerMyResolvConfInit,
   return HasFailure();
 }
 
+// Allow hosts path to be configurable
+NameContentList myhosts = {
+  {"/tmp/hosts", "10.0.12.26     foobar\n"
+                 "2001:A0:C::1A  foobar\n"}};
+CONTAINED_TEST_F(LibraryTest, ContainerMyHostsInit,
+                 "myhostname", "mydomain.org", myhosts) {
+  char filename[] = "/tmp/hosts";
+  ares_channel channel = nullptr;
+  struct ares_options options = {0};
+  options.hosts_path = strdup(filename);
+  int optmask = ARES_OPT_HOSTS_FILE;
+  EXPECT_EQ(ARES_SUCCESS, ares_init_options(&channel, &options, optmask));
+
+  optmask = 0;
+  free(options.hosts_path);
+  options.hosts_path = NULL;
+
+  EXPECT_EQ(ARES_SUCCESS, ares_save_options(channel, &options, &optmask));
+  EXPECT_EQ(ARES_OPT_HOSTS_FILE, (optmask & ARES_OPT_HOSTS_FILE));
+  EXPECT_EQ(std::string(filename), std::string(options.hosts_path));
+
+  ares_destroy_options(&options);
+  ares_destroy(channel);
+  return HasFailure();
+}
+
 NameContentList hostconf = {
   {"/etc/resolv.conf", "nameserver 1.2.3.4\n"
                        "sortlist1.2.3.4\n"  // malformed line
@@ -469,6 +519,24 @@ CONTAINED_TEST_F(LibraryTest, ContainerSvcConfInit,
   return HasFailure();
 }
 
+NameContentList malformedresolvconflookup = {
+  {"/etc/resolv.conf", "nameserver 1.2.3.4\n"
+                       "lookup garbage\n"}};  // malformed line
+CONTAINED_TEST_F(LibraryTest, ContainerMalformedResolvConfLookup,
+                 "myhostname", "mydomainname.org", malformedresolvconflookup) {
+  ares_channel channel = nullptr;
+  EXPECT_EQ(ARES_SUCCESS, ares_init(&channel));
+
+  struct ares_options opts;
+  int optmask = 0;
+  ares_save_options(channel, &opts, &optmask);
+  EXPECT_EQ(std::string("fb"), std::string(opts.lookups));
+  ares_destroy_options(&opts);
+
+  ares_destroy(channel);
+  return HasFailure();
+}
+
 // Failures when expected config filenames are inaccessible.
 class MakeUnreadable {
  public:
