@@ -1,3 +1,20 @@
+/*
+ * Copyright (C) The c-ares project
+ *
+ * Permission to use, copy, modify, and distribute this
+ * software and its documentation for any purpose and without
+ * fee is hereby granted, provided that the above copyright
+ * notice appear in all copies and that both that copyright
+ * notice and this permission notice appear in supporting
+ * documentation, and that the name of M.I.T. not be used in
+ * advertising or publicity pertaining to distribution of the
+ * software without specific, written prior permission.
+ * M.I.T. makes no representations about the suitability of
+ * this software for any purpose.  It is provided "as is"
+ * without express or implied warranty.
+ *
+ * SPDX-License-Identifier: MIT
+ */
 // This file includes tests that attempt to do real lookups
 // of DNS names using the local machine's live infrastructure.
 // As a result, we don't check the results very closely, to allow
@@ -18,6 +35,70 @@ unsigned char gdns_addr4[4] = {0x08, 0x08, 0x08, 0x08};
 unsigned char gdns_addr6[16] = {0x20, 0x01, 0x48, 0x60, 0x48, 0x60, 0x00, 0x00,
                                       0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x88, 0x88};
 
+MATCHER_P(IncludesAtLeastNumAddresses, n, "") {
+  if(!arg)
+    return false;
+  int cnt = 0;
+  for (const ares_addrinfo_node* ai = arg->nodes; ai != NULL; ai = ai->ai_next)
+    cnt++;
+  return cnt >= n;
+}
+
+MATCHER_P(OnlyIncludesAddrType, addrtype, "") {
+  if(!arg)
+    return false;
+  for (const ares_addrinfo_node* ai = arg->nodes; ai != NULL; ai = ai->ai_next)
+    if (ai->ai_family != addrtype)
+      return false;
+  return true;
+}
+
+MATCHER_P(IncludesAddrType, addrtype, "") {
+  if(!arg)
+    return false;
+  for (const ares_addrinfo_node* ai = arg->nodes; ai != NULL; ai = ai->ai_next)
+    if (ai->ai_family == addrtype)
+      return true;
+  return false;
+}
+
+//VIRT_NONVIRT_TEST_F(DefaultChannelTest, LiveGetAddrInfoV4) {
+  //struct ares_addrinfo_hints hints = {};
+  //hints.ai_family = AF_INET;
+  //AddrInfoResult result;
+  //ares_getaddrinfo(channel_, "www.google.com.", NULL, &hints, AddrInfoCallback, &result);
+  //Process();
+  //EXPECT_TRUE(result.done_);
+  //EXPECT_EQ(ARES_SUCCESS, result.status_);
+  //EXPECT_THAT(result.ai_, IncludesAtLeastNumAddresses(1));
+  //EXPECT_THAT(result.ai_, OnlyIncludesAddrType(AF_INET));
+//}
+
+//VIRT_NONVIRT_TEST_F(DefaultChannelTest, LiveGetAddrInfoV6) {
+  //struct ares_addrinfo_hints hints = {};
+  //hints.ai_family = AF_INET6;
+  //AddrInfoResult result;
+  //ares_getaddrinfo(channel_, "www.google.com.", NULL, &hints, AddrInfoCallback, &result);
+  //Process();
+  //EXPECT_TRUE(result.done_);
+  //EXPECT_EQ(ARES_SUCCESS, result.status_);
+  //EXPECT_THAT(result.ai_, IncludesAtLeastNumAddresses(1));
+  //EXPECT_THAT(result.ai_, OnlyIncludesAddrType(AF_INET6));
+//}
+
+//VIRT_NONVIRT_TEST_F(DefaultChannelTest, LiveGetAddrInfoUnspec) {
+  //struct ares_addrinfo_hints hints = {};
+  //hints.ai_family = AF_UNSPEC;
+  //AddrInfoResult result;
+  //ares_getaddrinfo(channel_, "www.google.com.", NULL, &hints, AddrInfoCallback, &result);
+  //Process();
+  //EXPECT_TRUE(result.done_);
+  //EXPECT_EQ(ARES_SUCCESS, result.status_);
+  //EXPECT_THAT(result.ai_, IncludesAtLeastNumAddresses(2));
+  //EXPECT_THAT(result.ai_, IncludesAddrType(AF_INET6));
+  //EXPECT_THAT(result.ai_, IncludesAddrType(AF_INET));
+//}
+
 VIRT_NONVIRT_TEST_F(DefaultChannelTest, LiveGetHostByNameV4) {
   HostResult result;
   ares_gethostbyname(channel_, "www.google.com.", AF_INET, HostCallback, &result);
@@ -81,7 +162,7 @@ TEST_P(DefaultChannelModeTest, LiveGetLocalhostByNameV4) {
   ares_gethostbyname(channel_, "localhost", AF_INET, HostCallback, &result);
   Process();
   EXPECT_TRUE(result.done_);
-  if ((result.status_ != ARES_ENOTFOUND) && (result.status_ != ARES_ECONNREFUSED)) {
+  if (result.status_ != ARES_ECONNREFUSED) {
     EXPECT_EQ(ARES_SUCCESS, result.status_);
     EXPECT_EQ(1, (int)result.host_.addrs_.size());
     EXPECT_EQ(AF_INET, result.host_.addrtype_);
@@ -94,7 +175,8 @@ TEST_P(DefaultChannelModeTest, LiveGetLocalhostByNameV6) {
   ares_gethostbyname(channel_, "localhost", AF_INET6, HostCallback, &result);
   Process();
   EXPECT_TRUE(result.done_);
-  if (result.status_ == ARES_SUCCESS) {
+  if (result.status_ != ARES_ECONNREFUSED) {
+    EXPECT_EQ(ARES_SUCCESS, result.status_);
     EXPECT_EQ(1, (int)result.host_.addrs_.size());
     EXPECT_EQ(AF_INET6, result.host_.addrtype_);
     std::stringstream ss;
@@ -103,6 +185,34 @@ TEST_P(DefaultChannelModeTest, LiveGetLocalhostByNameV6) {
   }
 }
 
+TEST_P(DefaultChannelModeTest, LiveGetNonExistLocalhostByNameV4) {
+  HostResult result;
+  ares_gethostbyname(channel_, "idonotexist.localhost", AF_INET, HostCallback, &result);
+  Process();
+  EXPECT_TRUE(result.done_);
+  if (result.status_ != ARES_ECONNREFUSED) {
+    EXPECT_EQ(ARES_SUCCESS, result.status_);
+    EXPECT_EQ(1, (int)result.host_.addrs_.size());
+    EXPECT_EQ(AF_INET, result.host_.addrtype_);
+    EXPECT_NE(std::string::npos, result.host_.name_.find("idonotexist.localhost"));
+  }
+}
+
+TEST_P(DefaultChannelModeTest, LiveGetNonExistLocalhostByNameV6) {
+  HostResult result;
+  ares_gethostbyname(channel_, "idonotexist.localhost", AF_INET6, HostCallback, &result);
+  Process();
+  EXPECT_TRUE(result.done_);
+  if (result.status_ != ARES_ECONNREFUSED) {
+    EXPECT_EQ(ARES_SUCCESS, result.status_);
+    EXPECT_EQ(1, (int)result.host_.addrs_.size());
+    EXPECT_EQ(AF_INET6, result.host_.addrtype_);
+    std::stringstream ss;
+    ss << HostEnt(result.host_);
+    EXPECT_NE(std::string::npos, result.host_.name_.find("idonotexist.localhost"));
+  }
+}
+
 TEST_P(DefaultChannelModeTest, LiveGetLocalhostByNameIPV4) {
   HostResult result;
   ares_gethostbyname(channel_, "127.0.0.1", AF_INET, HostCallback, &result);
@@ -150,8 +260,11 @@ TEST_P(DefaultChannelModeTest, LiveGetLocalhostByAddrV4) {
     EXPECT_EQ(ARES_SUCCESS, result.status_);
     EXPECT_LT(0, (int)result.host_.addrs_.size());
     EXPECT_EQ(AF_INET, result.host_.addrtype_);
-    EXPECT_NE(std::string::npos,
-              result.host_.name_.find("localhost"));
+    // oddly, travis does not resolve to localhost, but a random hostname starting with travis-job
+    if (result.host_.name_.find("travis-job") == std::string::npos) {
+        EXPECT_NE(std::string::npos,
+                  result.host_.name_.find("localhost"));
+    }
   }
 }
 
@@ -167,8 +280,9 @@ TEST_P(DefaultChannelModeTest, LiveGetLocalhostByAddrV6) {
     EXPECT_EQ(ARES_SUCCESS, result.status_);
     EXPECT_LT(0, (int)result.host_.addrs_.size());
     EXPECT_EQ(AF_INET6, result.host_.addrtype_);
-    EXPECT_NE(std::string::npos,
-              result.host_.name_.find("localhost"));
+    const std::string& name = result.host_.name_;
+    EXPECT_TRUE(std::string::npos != name.find("localhost") ||
+                std::string::npos != name.find("ip6-loopback"));
   }
 }
 
@@ -200,12 +314,12 @@ TEST_P(DefaultChannelModeTest, LiveGetHostByAddrFailAlloc) {
   EXPECT_EQ(ARES_ENOMEM, result.status_);
 }
 
-INSTANTIATE_TEST_CASE_P(Modes, DefaultChannelModeTest,
+INSTANTIATE_TEST_SUITE_P(Modes, DefaultChannelModeTest,
                         ::testing::Values("f", "b", "fb", "bf"));
 
 VIRT_NONVIRT_TEST_F(DefaultChannelTest, LiveSearchA) {
   SearchResult result;
-  ares_search(channel_, "www.youtube.com.", ns_c_in, ns_t_a,
+  ares_search(channel_, "www.youtube.com.", C_IN, T_A,
               SearchCallback, &result);
   Process();
   EXPECT_TRUE(result.done_);
@@ -214,7 +328,7 @@ VIRT_NONVIRT_TEST_F(DefaultChannelTest, LiveSearchA) {
 
 VIRT_NONVIRT_TEST_F(DefaultChannelTest, LiveSearchEmptyA) {
   SearchResult result;
-  ares_search(channel_, "", ns_c_in, ns_t_a,
+  ares_search(channel_, "", C_IN, T_A,
               SearchCallback, &result);
   Process();
   EXPECT_TRUE(result.done_);
@@ -223,7 +337,7 @@ VIRT_NONVIRT_TEST_F(DefaultChannelTest, LiveSearchEmptyA) {
 
 VIRT_NONVIRT_TEST_F(DefaultChannelTest, LiveSearchNS) {
   SearchResult result;
-  ares_search(channel_, "google.com.", ns_c_in, ns_t_ns,
+  ares_search(channel_, "google.com.", C_IN, T_NS,
               SearchCallback, &result);
   Process();
   EXPECT_TRUE(result.done_);
@@ -232,7 +346,7 @@ VIRT_NONVIRT_TEST_F(DefaultChannelTest, LiveSearchNS) {
 
 VIRT_NONVIRT_TEST_F(DefaultChannelTest, LiveSearchMX) {
   SearchResult result;
-  ares_search(channel_, "google.com.", ns_c_in, ns_t_mx,
+  ares_search(channel_, "google.com.", C_IN, T_MX,
               SearchCallback, &result);
   Process();
   EXPECT_TRUE(result.done_);
@@ -241,7 +355,7 @@ VIRT_NONVIRT_TEST_F(DefaultChannelTest, LiveSearchMX) {
 
 VIRT_NONVIRT_TEST_F(DefaultChannelTest, LiveSearchTXT) {
   SearchResult result;
-  ares_search(channel_, "google.com.", ns_c_in, ns_t_txt,
+  ares_search(channel_, "google.com.", C_IN, T_TXT,
               SearchCallback, &result);
   Process();
   EXPECT_TRUE(result.done_);
@@ -250,7 +364,7 @@ VIRT_NONVIRT_TEST_F(DefaultChannelTest, LiveSearchTXT) {
 
 VIRT_NONVIRT_TEST_F(DefaultChannelTest, LiveSearchSOA) {
   SearchResult result;
-  ares_search(channel_, "google.com.", ns_c_in, ns_t_soa,
+  ares_search(channel_, "google.com.", C_IN, T_SOA,
               SearchCallback, &result);
   Process();
   EXPECT_TRUE(result.done_);
@@ -259,7 +373,7 @@ VIRT_NONVIRT_TEST_F(DefaultChannelTest, LiveSearchSOA) {
 
 VIRT_NONVIRT_TEST_F(DefaultChannelTest, LiveSearchSRV) {
   SearchResult result;
-  ares_search(channel_, "_imap._tcp.gmail.com.", ns_c_in, ns_t_srv,
+  ares_search(channel_, "_imap._tcp.gmail.com.", C_IN, T_SRV,
               SearchCallback, &result);
   Process();
   EXPECT_TRUE(result.done_);
@@ -268,7 +382,7 @@ VIRT_NONVIRT_TEST_F(DefaultChannelTest, LiveSearchSRV) {
 
 VIRT_NONVIRT_TEST_F(DefaultChannelTest, LiveSearchANY) {
   SearchResult result;
-  ares_search(channel_, "google.com.", ns_c_in, ns_t_any,
+  ares_search(channel_, "google.com.", C_IN, T_ANY,
               SearchCallback, &result);
   Process();
   EXPECT_TRUE(result.done_);
@@ -628,6 +742,54 @@ TEST_F(DefaultChannelTest, VerifySocketFunctionCallback) {
 
 }
 
+TEST_F(DefaultChannelTest, LiveSetServers) {
+  struct ares_addr_node server1;
+  struct ares_addr_node server2;
+  server1.next = &server2;
+  server1.family = AF_INET;
+  server1.addr.addr4.s_addr = htonl(0x01020304);
+  server2.next = nullptr;
+  server2.family = AF_INET;
+  server2.addr.addr4.s_addr = htonl(0x02030405);
+
+  // Change not allowed while request is pending
+  HostResult result;
+  ares_gethostbyname(channel_, "www.google.com.", AF_INET, HostCallback, &result);
+  EXPECT_EQ(ARES_ENOTIMP, ares_set_servers(channel_, &server1));
+  ares_cancel(channel_);
+}
+
+TEST_F(DefaultChannelTest, LiveSetServersPorts) {
+  struct ares_addr_port_node server1;
+  struct ares_addr_port_node server2;
+  server1.next = &server2;
+  server1.family = AF_INET;
+  server1.addr.addr4.s_addr = htonl(0x01020304);
+  server1.udp_port = 111;
+  server1.tcp_port = 111;
+  server2.next = nullptr;
+  server2.family = AF_INET;
+  server2.addr.addr4.s_addr = htonl(0x02030405);
+  server2.udp_port = 0;
+  server2.tcp_port = 0;;
+  EXPECT_EQ(ARES_ENODATA, ares_set_servers_ports(nullptr, &server1));
+
+  // Change not allowed while request is pending
+  HostResult result;
+  ares_gethostbyname(channel_, "www.google.com.", AF_INET, HostCallback, &result);
+  EXPECT_EQ(ARES_ENOTIMP, ares_set_servers_ports(channel_, &server1));
+  ares_cancel(channel_);
+}
+
+TEST_F(DefaultChannelTest, LiveSetServersCSV) {
+  // Change not allowed while request is pending
+  HostResult result;
+  ares_gethostbyname(channel_, "www.google.com.", AF_INET, HostCallback, &result);
+  EXPECT_EQ(ARES_ENOTIMP, ares_set_servers_csv(channel_, "1.2.3.4,2.3.4.5"));
+  EXPECT_EQ(ARES_ENOTIMP, ares_set_servers_ports_csv(channel_, "1.2.3.4:56,2.3.4.5:67"));
+  ares_cancel(channel_);
+}
+
 
 }  // namespace test
 }  // namespace ares
