@@ -1,3 +1,20 @@
+/*
+ * Copyright (C) The c-ares project
+ *
+ * Permission to use, copy, modify, and distribute this
+ * software and its documentation for any purpose and without
+ * fee is hereby granted, provided that the above copyright
+ * notice appear in all copies and that both that copyright
+ * notice and this permission notice appear in supporting
+ * documentation, and that the name of M.I.T. not be used in
+ * advertising or publicity pertaining to distribution of the
+ * software without specific, written prior permission.
+ * M.I.T. makes no representations about the suitability of
+ * this software for any purpose.  It is provided "as is"
+ * without express or implied warranty.
+ *
+ * SPDX-License-Identifier: MIT
+ */
 #include "ares-test.h"
 #include "dns-proto.h"
 
@@ -45,12 +62,6 @@ TEST_F(DefaultChannelTest, SetServers) {
   EXPECT_EQ(ARES_SUCCESS, ares_set_servers(channel_, &server1));
   std::vector<std::string> expected = {"1.2.3.4", "2.3.4.5"};
   EXPECT_EQ(expected, GetNameServers(channel_));
-
-  // Change not allowed while request is pending
-  HostResult result;
-  ares_gethostbyname(channel_, "www.google.com.", AF_INET, HostCallback, &result);
-  EXPECT_EQ(ARES_ENOTIMP, ares_set_servers(channel_, &server1));
-  ares_cancel(channel_);
 }
 
 TEST_F(DefaultChannelTest, SetServersPorts) {
@@ -75,12 +86,6 @@ TEST_F(DefaultChannelTest, SetServersPorts) {
   EXPECT_EQ(ARES_SUCCESS, ares_set_servers_ports(channel_, &server1));
   std::vector<std::string> expected = {"1.2.3.4:111", "2.3.4.5"};
   EXPECT_EQ(expected, GetNameServers(channel_));
-
-  // Change not allowed while request is pending
-  HostResult result;
-  ares_gethostbyname(channel_, "www.google.com.", AF_INET, HostCallback, &result);
-  EXPECT_EQ(ARES_ENOTIMP, ares_set_servers_ports(channel_, &server1));
-  ares_cancel(channel_);
 }
 
 TEST_F(DefaultChannelTest, SetServersCSV) {
@@ -108,13 +113,6 @@ TEST_F(DefaultChannelTest, SetServersCSV) {
   std::vector<std::string> expected2 = {"1.2.3.4:54", "[0102:0304:0506:0708:0910:1112:1314:1516]:80", "2.3.4.5:55"};
   EXPECT_EQ(expected2, GetNameServers(channel_));
 
-  // Change not allowed while request is pending
-  HostResult result;
-  ares_gethostbyname(channel_, "www.google.com.", AF_INET, HostCallback, &result);
-  EXPECT_EQ(ARES_ENOTIMP, ares_set_servers_csv(channel_, "1.2.3.4,2.3.4.5"));
-  EXPECT_EQ(ARES_ENOTIMP, ares_set_servers_ports_csv(channel_, "1.2.3.4:56,2.3.4.5:67"));
-  ares_cancel(channel_);
-
   // Should survive duplication
   ares_channel channel2;
   EXPECT_EQ(ARES_SUCCESS, ares_dup(&channel2, channel_));
@@ -185,13 +183,13 @@ TEST_F(LibraryTest, InetNtoP) {
 TEST_F(LibraryTest, Mkquery) {
   byte* p;
   int len;
-  ares_mkquery("example.com", ns_c_in, ns_t_a, 0x1234, 0, &p, &len);
+  ares_mkquery("example.com", C_IN, T_A, 0x1234, 0, &p, &len);
   std::vector<byte> data(p, p + len);
   ares_free_string(p);
 
   std::string actual = PacketToString(data);
   DNSPacket pkt;
-  pkt.set_qid(0x1234).add_question(new DNSQuestion("example.com", ns_t_a));
+  pkt.set_qid(0x1234).add_question(new DNSQuestion("example.com", T_A));
   std::string expected = PacketToString(pkt.data());
   EXPECT_EQ(expected, actual);
 }
@@ -200,14 +198,14 @@ TEST_F(LibraryTest, CreateQuery) {
   byte* p;
   int len;
   EXPECT_EQ(ARES_SUCCESS,
-            ares_create_query("exam\\@le.com", ns_c_in, ns_t_a, 0x1234, 0,
+            ares_create_query("exam\\@le.com", C_IN, T_A, 0x1234, 0,
                               &p, &len, 0));
   std::vector<byte> data(p, p + len);
   ares_free_string(p);
 
   std::string actual = PacketToString(data);
   DNSPacket pkt;
-  pkt.set_qid(0x1234).add_question(new DNSQuestion("exam@le.com", ns_t_a));
+  pkt.set_qid(0x1234).add_question(new DNSQuestion("exam@le.com", T_A));
   std::string expected = PacketToString(pkt.data());
   EXPECT_EQ(expected, actual);
 }
@@ -216,7 +214,7 @@ TEST_F(LibraryTest, CreateQueryTrailingEscapedDot) {
   byte* p;
   int len;
   EXPECT_EQ(ARES_SUCCESS,
-            ares_create_query("example.com\\.", ns_c_in, ns_t_a, 0x1234, 0,
+            ares_create_query("example.com\\.", C_IN, T_A, 0x1234, 0,
                               &p, &len, 0));
   std::vector<byte> data(p, p + len);
   ares_free_string(p);
@@ -234,7 +232,7 @@ TEST_F(LibraryTest, CreateQueryNameTooLong) {
               "a1234567890123456789.b1234567890123456789.c1234567890123456789.d1234567890123456789."
               "a1234567890123456789.b1234567890123456789.c1234567890123456789.d1234567890123456789."
               "x1234567890123456789.y1234567890123456789.",
-              ns_c_in, ns_t_a, 0x1234, 0, &p, &len, 0));
+              C_IN, T_A, 0x1234, 0, &p, &len, 0));
 }
 
 TEST_F(LibraryTest, CreateQueryFailures) {
@@ -247,7 +245,7 @@ TEST_F(LibraryTest, CreateQueryFailures) {
   }
   p = nullptr;
   EXPECT_EQ(ARES_EBADNAME,
-            ares_create_query(longname.c_str(), ns_c_in, ns_t_a, 0x1234, 0,
+            ares_create_query(longname.c_str(), C_IN, T_A, 0x1234, 0,
                     &p, &len, 0));
   if (p) ares_free_string(p);
 
@@ -255,7 +253,7 @@ TEST_F(LibraryTest, CreateQueryFailures) {
 
   p = nullptr;
   EXPECT_EQ(ARES_ENOMEM,
-            ares_create_query("example.com", ns_c_in, ns_t_a, 0x1234, 0,
+            ares_create_query("example.com", C_IN, T_A, 0x1234, 0,
                     &p, &len, 0));
   if (p) ares_free_string(p);
 
@@ -263,14 +261,14 @@ TEST_F(LibraryTest, CreateQueryFailures) {
   std::string longlabel = "a.a123456789b123456789c123456789d123456789e123456789f123456789g123456789.org";
   p = nullptr;
   EXPECT_EQ(ARES_EBADNAME,
-            ares_create_query(longlabel.c_str(), ns_c_in, ns_t_a, 0x1234, 0,
+            ares_create_query(longlabel.c_str(), C_IN, T_A, 0x1234, 0,
                     &p, &len, 0));
   if (p) ares_free_string(p);
 
   // Empty non-terminal label
   p = nullptr;
   EXPECT_EQ(ARES_EBADNAME,
-            ares_create_query("example..com", ns_c_in, ns_t_a, 0x1234, 0,
+            ares_create_query("example..com", C_IN, T_A, 0x1234, 0,
                     &p, &len, 0));
   if (p) ares_free_string(p);
 }
@@ -279,7 +277,7 @@ TEST_F(LibraryTest, CreateQueryOnionDomain) {
   byte* p;
   int len;
   EXPECT_EQ(ARES_ENOTFOUND,
-            ares_create_query("dontleak.onion", ns_c_in, ns_t_a, 0x1234, 0,
+            ares_create_query("dontleak.onion", C_IN, T_A, 0x1234, 0,
                               &p, &len, 0));
 }
 
@@ -296,19 +294,28 @@ TEST_F(DefaultChannelTest, HostByNameFileOnionDomain) {
             ares_gethostbyname_file(channel_, "dontleak.onion", AF_INET, &h));
 }
 
+TEST_F(DefaultChannelTest, GetAddrinfoOnionDomain) {
+  AddrInfoResult result;
+  struct ares_addrinfo_hints hints = {};
+  hints.ai_family = AF_UNSPEC;
+  ares_getaddrinfo(channel_, "dontleak.onion", NULL, &hints, AddrInfoCallback, &result);
+  EXPECT_TRUE(result.done_);
+  EXPECT_EQ(ARES_ENOTFOUND, result.status_);
+}
+
 // Interesting question: should tacking on a search domain let the query
 // through? It seems safer to reject it because "supersecret.onion.search"
 // still leaks information about the query to malicious resolvers.
 TEST_F(DefaultChannelTest, SearchOnionDomain) {
   SearchResult result;
-  ares_search(channel_, "dontleak.onion", ns_c_in, ns_t_a,
+  ares_search(channel_, "dontleak.onion", C_IN, T_A,
               SearchCallback, &result);
   EXPECT_TRUE(result.done_);
   EXPECT_EQ(ARES_ENOTFOUND, result.status_);
 }
 
 TEST_F(DefaultChannelTest, SendFailure) {
-  unsigned char buf[2];
+  unsigned char buf[2] = {};
   SearchResult result;
   ares_send(channel_, buf, sizeof(buf), SearchCallback, &result);
   EXPECT_TRUE(result.done_);
@@ -464,14 +471,14 @@ TEST_F(LibraryTest, CreateEDNSQuery) {
   byte* p;
   int len;
   EXPECT_EQ(ARES_SUCCESS,
-            ares_create_query("example.com", ns_c_in, ns_t_a, 0x1234, 0,
+            ares_create_query("example.com", C_IN, T_A, 0x1234, 0,
                               &p, &len, 1280));
   std::vector<byte> data(p, p + len);
   ares_free_string(p);
 
   std::string actual = PacketToString(data);
   DNSPacket pkt;
-  pkt.set_qid(0x1234).add_question(new DNSQuestion("example.com", ns_t_a))
+  pkt.set_qid(0x1234).add_question(new DNSQuestion("example.com", T_A))
     .add_additional(new DNSOptRR(0, 1280));
   std::string expected = PacketToString(pkt.data());
   EXPECT_EQ(expected, actual);
@@ -480,13 +487,13 @@ TEST_F(LibraryTest, CreateEDNSQuery) {
 TEST_F(LibraryTest, CreateRootQuery) {
   byte* p;
   int len;
-  ares_create_query(".", ns_c_in, ns_t_a, 0x1234, 0, &p, &len, 0);
+  ares_create_query(".", C_IN, T_A, 0x1234, 0, &p, &len, 0);
   std::vector<byte> data(p, p + len);
   ares_free_string(p);
 
   std::string actual = PacketToString(data);
   DNSPacket pkt;
-  pkt.set_qid(0x1234).add_question(new DNSQuestion("", ns_t_a));
+  pkt.set_qid(0x1234).add_question(new DNSQuestion("", T_A));
   std::string expected = PacketToString(pkt.data());
   EXPECT_EQ(expected, actual);
 }
